@@ -314,6 +314,9 @@ static BOOL debugLeaks;
     {
       fm = [GCSFolderManager defaultFolderManager];
 
+      // Create the sessions table
+      [[fm adminFolder] createFolderIfNotExists];
+
       // Create the sessions table
       [[fm sessionsFolder] createFolderIfNotExists];
       
@@ -349,8 +352,10 @@ static BOOL debugLeaks;
     {
       if ([[context request] handledByDefaultHandler])
         authenticator = [SOGoWebAuthenticator sharedSOGoWebAuthenticator];
-      else
+      else {
         authenticator = [SOGoDAVAuthenticator sharedSOGoDAVAuthenticator];
+      }
+        
     }
 
   return authenticator;
@@ -363,10 +368,16 @@ static BOOL debugLeaks;
 {
   SOGoUser *user;
   id userFolder;
+  NSData *decodedLogin;
+  NSString *login;
+  WORequest *request;
 
-  user = [SOGoUser userWithLogin: _key roles: nil];
+  request = [_ctx request];
+  login = [SOGoUser getDecryptedUsernameIfNeeded: _key request: request];
+
+  user = [SOGoUser userWithLogin: login roles: nil];
   if (user)
-    userFolder = [$(@"SOGoUserFolder") objectWithName: _key
+    userFolder = [$(@"SOGoUserFolder") objectWithName: login
                                           inContainer: self];
   else
     userFolder = nil;
