@@ -239,6 +239,13 @@
   [newRecord setObject: [oldRecord objectForKey: @"c_uid"]
                 forKey: @"c_uid"];
 
+  //source id
+  [newRecord setObject: [source sourceID] forKey: @"sourceID"];
+
+  //source display name
+  [newRecord setObject: [self displayName]
+                          forKey: @"containerName"];
+
   // c_name => id
   [newRecord setObject: [oldRecord objectForKey: @"c_name"]
                 forKey: @"c_name"];
@@ -247,12 +254,11 @@
 
   // displayname || c_cn => fn
   data = [oldRecord objectForKey: @"displayname"];
-  if (!data)
+  if(!data || data == [NSNull null] || [data length] < 1)
     data = [oldRecord objectForKey: @"c_cn"];
-  if (data)
-    [newRecord setObject: data forKey: @"fn"];
-  else
+  if(!data || data == [NSNull null] || [data length] < 1)
     data = @"";
+  [newRecord setObject: data forKey: @"fn"];
   [newRecord setObject: data forKey: @"c_cn"];
 
   // sn => c_sn
@@ -427,26 +433,43 @@
 {
   NSArray *records, *result;
   EOSortOrdering *ordering;
+  BOOL processed;
 
   result = nil;
+  processed = NO;
 
-  if ([filter length] > 0 || ![source listRequiresDot])
+  [source setListRequiresDot: NO];
+  if ( ![source listRequiresDot])
     {
-      records = [source fetchContactsMatching: filter
+      if ([filter length] > 0) {
+        records = [source fetchContactsMatching: filter
                                  withCriteria: criteria
                                      inDomain: domain];
-      [childRecords setObjects: records
-                       forKeys: [records objectsForKey: @"c_name"
-                                        notFoundMarker: nil]];
-      records = [self _flattenedRecords: records];
-      ordering
-        = [EOSortOrdering sortOrderingWithKey: sortKey
-                          selector: ((sortOrdering == NSOrderedDescending)
-                                     ? EOCompareCaseInsensitiveDescending
-                                     : EOCompareCaseInsensitiveAscending)];
-      result
-        = [records sortedArrayUsingKeyOrderArray:
-                     [NSArray arrayWithObject: ordering]];
+        processed = YES;
+      } else if ([[SOGoSystemDefaults sharedSystemDefaults] globalAddressBookFirstEntriesEnabled]) {
+        records = [source fetchContactsMatching: filter
+                                 withCriteria: criteria
+                                     inDomain: domain
+                                        limit: [[SOGoSystemDefaults sharedSystemDefaults] globalAddressBookFirstEntriesCount]];
+        processed = YES;
+      }
+
+      if (processed) {
+        [childRecords setObjects: records
+                        forKeys: [records objectsForKey: @"c_name"
+                                          notFoundMarker: nil]];
+        records = [self _flattenedRecords: records];
+        ordering
+          = [EOSortOrdering sortOrderingWithKey: sortKey
+                            selector: ((sortOrdering == NSOrderedDescending)
+                                      ? EOCompareCaseInsensitiveDescending
+                                      : EOCompareCaseInsensitiveAscending)];
+        result
+          = [records sortedArrayUsingKeyOrderArray:
+                      [NSArray arrayWithObject: ordering]];
+      }
+      
+      
     }
 
   return result;
