@@ -67,6 +67,7 @@
 #import <SOGo/SOGoUserFolder.h>
 #import <SOGo/SOGoUserDefaults.h>
 #import <SOGo/SOGoSystemDefaults.h>
+#import <SOGo/NGMimeFileData+SOGo.h>
 
 #import <NGCards/NGVCard.h>
 
@@ -138,9 +139,11 @@ static NSString    *userAgent      = nil;
       sourceURL = nil;
       sourceFlag = nil;
       inReplyTo = nil;
+      references = nil;
       isHTML = NO;
       sign = NO;
       encrypt = NO;
+      tmpFiles = [[NSMutableArray alloc] init];
     }
 
   return self;
@@ -155,6 +158,8 @@ static NSString    *userAgent      = nil;
   [sourceURL release];
   [sourceFlag release];
   [inReplyTo release];
+  [references release];
+  [tmpFiles release];
   [super dealloc];
 }
 
@@ -361,6 +366,16 @@ static NSString    *userAgent      = nil;
   ASSIGN (inReplyTo, newInReplyTo);
 }
 
+- (NSString *) references
+{
+  return references;
+}
+
+- (void) setReferences: (NSString *) newReferences
+{
+  ASSIGN (references, newReferences);
+}
+
 - (void) setSourceURL: (NSString *) newSourceURL
 {
   ASSIGN (sourceURL, newSourceURL);
@@ -421,6 +436,8 @@ static NSString    *userAgent      = nil;
                 forKey: @"isHTML"];
       if (inReplyTo)
 	[infos setObject: inReplyTo forKey: @"inReplyTo"];
+      if (references)
+	[infos setObject: references forKey: @"references"];
       if (sourceIMAP4ID > -1)
 	[infos setObject: [NSString stringWithFormat: @"%i", sourceIMAP4ID]
                   forKey: @"sourceIMAP4ID"];
@@ -492,6 +509,10 @@ static NSString    *userAgent      = nil;
   value = [infoDict objectForKey: @"inReplyTo"];
   if (value)
     [self setInReplyTo: value];
+
+  value = [infoDict objectForKey: @"references"];
+  if (value)
+    [self setReferences: value];
 }
 
 //
@@ -570,7 +591,7 @@ static NSString    *userAgent      = nil;
   id result;
 
   error = nil;
-  message = [self mimeMessageForRecipient: nil];
+  message = [self mimeMessageForRecipient: nil extractingImages: NO];
 
   if (!message)
     {
@@ -914,11 +935,10 @@ static NSString    *userAgent      = nil;
       if (filename)
         {
           NSMutableDictionary *currentInfo;
-
           currentInfo = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                                filename, @"filename",
                                              mimeType, @"mimetype",
-                                             bodyId, @"bodyid", 
+                                             bodyId, @"bodyId", 
                                              nil];
           [self saveAttachment: body
                   withMetadata: currentInfo];
@@ -1039,12 +1059,13 @@ static NSString    *userAgent      = nil;
 - (void) fetchMailForReplying: (SOGoMailObject *) sourceMail
                         toAll: (BOOL) toAll
 {
-  BOOL fromSentMailbox;
+  BOOL fromSentMailbox, shouldRetrieveAttachements;
   NSString *msgID;
+  NSString *oldReferences;
   NSMutableDictionary *info;
   NGImap4Envelope *sourceEnvelope;
   SOGoUserDefaults *ud;
-
+  
   fromSentMailbox = [[sourceMail container] isKindOfClass: [SOGoSentFolder class]];
   [sourceMail fetchCoreInfos];
 
@@ -1059,8 +1080,16 @@ static NSString    *userAgent      = nil;
   msgID = [sourceEnvelope messageID];
   if ([msgID length] > 0)
     [self setInReplyTo: msgID];
+    oldReferences = [sourceMail references];
+    if ([oldReferences length] > 0)
+      [self setReferences: [oldReferences stringByAppendingFormat: @" %@", msgID]];
+    else
+      [self setReferences: msgID];
+    
+
 
   ud = [[context activeUser] userDefaults];
+  
 
   [self setText: [sourceMail contentForReply]];
   [self setHeaders: info];
@@ -1070,12 +1099,10 @@ static NSString    *userAgent      = nil;
   [self setSourceIMAP4ID: [[sourceMail nameInContainer] intValue]];
   [self setSourceFolderWithMailObject: sourceMail];
 
+  [self setText: [sourceMail contentForReply]];
+  shouldRetrieveAttachements = [[ud mailComposeMessageType] isEqualToString: @"html"];
 
-  ud = [[context activeUser] userDefaults];
-  // TODO: Change mailMessageForwarding for reply
-  if ([[ud mailMessageForwarding] isEqualToString: @"inline"])
-  {
-    [self setText: [sourceMail contentForReply]];
+  if (shouldRetrieveAttachements) {
     if ([sourceMail isEncrypted])
       [self _fetchAttachmentsFromEncryptedMail: sourceMail onlyImages: YES];
     else if ([sourceMail isOpaqueSigned])
@@ -1084,7 +1111,7 @@ static NSString    *userAgent      = nil;
       [self _fetchAttachmentsFromMail: sourceMail onlyImages: YES];
   }
 
-    [self save];
+  [self save];
 
   [self storeInfo];
 }
@@ -1135,7 +1162,7 @@ static NSString    *userAgent      = nil;
       // TODO: use subject for filename?
       // error = [newDraft saveAttachment:content withName:@"forward.eml"];
       signature = [[self mailAccountFolder] signature];
-      if ([signature length])
+      if ([signature length] && [ud mailUseSignatureOnForward])
         {
           nl = (isHTML ? @"<br />" : @"\n");
           space = (isHTML ? @"&nbsp;" : @" ");
@@ -1835,6 +1862,9 @@ static NSString    *userAgent      = nil;
   if (inReplyTo)
     [map setObject: inReplyTo forKey: @"in-reply-to"];
 
+  if (references)
+    [map setObject: references forKey: @"references"];
+
   /* add subject */
   if ([(s = [headers objectForKey: @"subject"]) length] > 0)
     [map setObject: [s asQPSubjectString: @"utf-8"]
@@ -1896,8 +1926,11 @@ static NSString    *userAgent      = nil;
   if (_extractImages)
     {
       newText = [text htmlByExtractingImages: extractedBodyParts];
-      if ([extractedBodyParts count])
+      if ([extractedBodyParts count]) {
         [self setText: newText];
+        [self addTmpFiles: extractedBodyParts];
+      }
+
     }
 
   map = [self mimeHeaderMapWithHeaders: _headers
@@ -1908,8 +1941,11 @@ static NSString    *userAgent      = nil;
       //[self debugWithFormat: @"MIME Envelope: %@", map];
       allBodyParts = [self bodyPartsForAllAttachments];
 
-      if (!allBodyParts)
-	return nil;
+      if (!allBodyParts) {
+        return nil;
+      } else {
+        [self addTmpFiles: allBodyParts];
+      }
 
       //[self debugWithFormat: @"attachments: %@", bodyParts];
 
@@ -1929,12 +1965,12 @@ static NSString    *userAgent      = nil;
     }
 
   return message;
-}
+} 
 
 //
 // Return a NGMimeMessage object with inline HTML images (<img src=data>) extracted as attachments (<img src=cid>).
 //
-- (NSData *) mimeMessageForRecipient: (NSString *) theRecipient
+- (NSData *) mimeMessageForRecipient: (NSString *) theRecipient extractingImages: (BOOL)extractImage
 {
   NGMimeMessageGenerator *generator, *partGenerator;
   NGMimeMessage *mimeMessage;
@@ -1947,7 +1983,7 @@ static NSString    *userAgent      = nil;
   // Nothing to sign or encrypt, let's generate the message and return immediately
   if (![self sign] && ![self encrypt])
     {
-      mimeMessage = [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: YES  bodyOnly: NO];
+      mimeMessage = [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: extractImage  bodyOnly: NO];
       if (mimeMessage)
         {
           generator = [[[NGMimeMessageGenerator alloc] init] autorelease];
@@ -1961,7 +1997,7 @@ static NSString    *userAgent      = nil;
   partGenerator = [[[NGMimePartGenerator alloc] init] autorelease];
   content = [partGenerator generateMimeFromPart: [self mimeMessageWithHeaders: nil
                                                                     excluding: nil
-                                                             extractingImages: YES
+                                                             extractingImages: extractImage
                                                                      bodyOnly: YES]];
 
   if ([self sign])
@@ -2199,6 +2235,41 @@ static NSString    *userAgent      = nil;
   return [self sendMailAndCopyToSent: YES];
 }
 
+// Extract tmp files in NGMimeBuildMimeTempDirectory from NSArray and store locally
+// The tmp files will be deleted once the message sent
+- (void) addTmpFiles:(NSArray *) parts {
+  NGMimeBodyPart *part;
+
+  for (part in parts) {
+    NGMimeFileData *body;
+    NSString *path;
+
+    body = [part body];
+    if (body && [body isKindOfClass: [NGMimeFileData class]]) {
+      path = [body path];
+      if (path) {
+        [tmpFiles addObject: path];
+      }
+    }
+  }
+}
+
+// Clean temporary files
+- (void) cleanTmpFiles {
+  NSString *path;
+  NSFileManager *fm;
+
+  fm = [NSFileManager defaultManager];
+  
+  for (path in tmpFiles) {
+    if ([fm fileExistsAtPath: path])
+      [fm removeFileAtPath: path handler: nil];
+  }
+
+  // Clean tmp files
+  [tmpFiles removeAllObjects];
+}
+
 //
 //
 //
@@ -2207,11 +2278,13 @@ static NSString    *userAgent      = nil;
   NSData *message, *messageForSent;
   SOGoMailFolder *sentFolder;
   SOGoDomainDefaults *dd;
-  NSURL *sourceIMAP4URL;
+  NSURL *sourceIMAP4URL, *smtpUrl;
   NSException *error;
+  NSString *userId;
 
   dd = [[context activeUser] domainDefaults];
   messageForSent = nil;
+  userId = [[self->container mailAccountFolder] nameInContainer];
 
   // If we are encrypting mails, let's generate and
   // send them individually
@@ -2228,47 +2301,79 @@ static NSString    *userAgent      = nil;
           recipient = [recipients objectAtIndex: i];
 
           if ([[context activeUser] hasEmail: recipient])
-            message = messageForSent = [self mimeMessageForRecipient: nil];
+            message = messageForSent = [self mimeMessageForRecipient: nil extractingImages: YES];
           else
-            message = [self mimeMessageForRecipient: recipient];
+            message = [self mimeMessageForRecipient: recipient extractingImages: YES];
 
           if (!message)
             return  [NSException exceptionWithHTTPStatus: 500
                                                   reason: @"could not generate message content"];
 
-          error = [[SOGoMailer mailerWithDomainDefaults: dd]
-                    sendMailData: message
-                    toRecipients: [NSArray arrayWithObject: recipient]
-                          sender: [self sender]
-                    withAuthenticator: [self authenticatorInContext: context]
-                       inContext: context
-                   systemMessage: NO];
+          smtpUrl = [self smtp4URL];
 
-          if (error)
+          if (smtpUrl)
+          {
+            error = [[SOGoMailer mailerWithDomainDefaultsAndSmtpUrl: dd smtpUrl: smtpUrl userIdAccount: userId]
+                        sendMailData: message
+                        toRecipients: [NSArray arrayWithObject: recipient]
+                              sender: [self sender]
+                   withAuthenticator: [self authenticatorInContext: context]
+                           inContext: context
+                       systemMessage: NO];
+          }
+          else
+          {
+            error = [[SOGoMailer mailerWithDomainDefaults: dd]
+                 sendMailData: message
+                 toRecipients: [NSArray arrayWithObject: recipient]
+                       sender: [self sender]
+            withAuthenticator: [self authenticatorInContext: context]
+                    inContext: context
+                systemMessage: NO];
+          }
+
+          if (error) {
+            [self cleanTmpFiles];
             return error;
+          }
         }
 
       // If the current user isn't part of the recipient list for encrypted emails
       // let's generate a crypted email for its sent folder.
       if (!messageForSent)
-        messageForSent = [self mimeMessageForRecipient: nil];
+        messageForSent = [self mimeMessageForRecipient: nil extractingImages: YES];
     }
   else
     {
       // Encryption is done or not, if we didn't have to.
-      message = messageForSent = [self mimeMessageForRecipient: nil];
+      message = messageForSent = [self mimeMessageForRecipient: nil extractingImages: YES];
 
       if (!message)
         return  [NSException exceptionWithHTTPStatus: 500
                                               reason: @"could not generate message content"];
 
-      error = [[SOGoMailer mailerWithDomainDefaults: dd]
-                  sendMailData: message
-                  toRecipients: [self allBareRecipients]
-                        sender: [self sender]
+      smtpUrl = [self smtp4URL];
+
+      if (smtpUrl)
+      {
+        error = [[SOGoMailer mailerWithDomainDefaultsAndSmtpUrl: dd smtpUrl: smtpUrl userIdAccount: userId]
+                    sendMailData: message
+                    toRecipients: [self allBareRecipients]
+                          sender: [self sender]
                 withAuthenticator: [self authenticatorInContext: context]
-                     inContext: context
-                 systemMessage: NO];
+                        inContext: context
+                    systemMessage: NO];
+      }
+      else
+      {
+        error = [[SOGoMailer mailerWithDomainDefaults: dd]
+              sendMailData: message
+              toRecipients: [self allBareRecipients]
+                    sender: [self sender]
+        withAuthenticator: [self authenticatorInContext: context]
+                inContext: context
+            systemMessage: NO];
+      }
     }
 
   if (!error && copyToSent)
@@ -2304,6 +2409,8 @@ static NSString    *userAgent      = nil;
       [imap4 doesMailboxExistAtURL: [container imap4URL]])
     [(SOGoDraftsFolder *) container expunge];
 
+  [self cleanTmpFiles];
+
   return error;
 }
 
@@ -2329,7 +2436,7 @@ static NSString    *userAgent      = nil;
   NSString *str;
   NSData *message;
 
-  message = [self mimeMessageForRecipient: nil];
+  message = [self mimeMessageForRecipient: nil extractingImages: NO];
   if (message)
     {
       str = [[NSString alloc] initWithData: message
