@@ -217,7 +217,7 @@
 
   v =  [self integerForKey: @"SOGoForwardConstraints"];
   
-  return (v > 2 ? 0 : v);
+  return (v > 3 ? 0 : v);
 }
 
 - (NSArray *) forwardConstraintsDomains
@@ -225,6 +225,11 @@
   return [self stringArrayForKey: @"SOGoForwardConstraintsDomains"];
 }
 
+- (BOOL) notificationEnabled
+{
+  return [self boolForKey: @"SOGoNotificationEnabled"];
+}
+
 - (BOOL) vacationEnabled
 {
   return [self boolForKey: @"SOGoVacationEnabled"];
@@ -443,4 +448,16 @@
   return [self boolForKey: @"SOGoCreateIdentitiesDisabled"];
 }
 
+- (NSString *) passwordRecoveryFrom
+{
+  NSString *emailFrom;
+
+  emailFrom = [self stringForKey: @"SOGoPasswordRecoveryFrom"];
+
+  if (!emailFrom)
+    emailFrom = @"noreply@domain.com"; // Default email from when no domain set
+
+  return emailFrom;
+}
+
 @end
