@@ -211,6 +211,11 @@
 
 /* ownership */
 
+- (NSString *) owner
+{
+  return owner;
+}
+
 - (void) setOwner: (NSString *) newOwner
 {
   NSString *uid;
@@ -941,6 +946,7 @@
   NSMutableArray *path;
   NSString *baseURL, *urlMethod, *fullHost;
   NSNumber *port;
+  int i;
 
   serverURL = [context serverURL];
   baseURL = [[self baseURLInContext: context] stringByUnescapingURL];
@@ -968,6 +974,17 @@
   else
     fullHost = [serverURL host];
 
+  for (i = 0 ; i < [path count] ; i++) {
+    // For DAV url, the username must be decrypted
+    // Username is placed afted dav in url components
+    if ([[[path objectAtIndex: i] lowercaseString] isEqualToString:@"dav"]) {
+      if ([path count] > (i + 1)) {
+        [path replaceObjectAtIndex: (i +1)
+                        withObject: [SOGoUser getDecryptedUsernameIfNeeded: [path objectAtIndex: (i + 1)]  request: [context request]]];
+      }
+    }
+  }
+
   url = [[NSURL alloc] initWithScheme: [serverURL scheme]
 		       host: fullHost
 		       path: [path componentsJoinedByString: @"/"]];
