@@ -25,10 +25,12 @@
 #import <Foundation/NSFileManager.h>
 #import <Foundation/NSFileManager.h>
 #import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSProcessInfo.h>
 
 #import <NGExtensions/NSObject+Logs.h>
 
 #import "NSArray+Utilities.h"
+#import "NSString+Crypto.h"
 #import "NSDictionary+Utilities.h"
 #import "SOGoStartupLogger.h"
 
@@ -353,6 +355,60 @@ _injectConfigurationFromFile (NSMutableDictionary *defaultsDict,
   return [self stringForKey: @"SOGoEncryptionKey"];
 }
 
+
+- (BOOL) isSogoSecretSet
+{
+  NSString *type;
+  type = [self stringForKey: @"SOGoSecretType"];
+  if(!type || [type isEqualToString:@"none"])
+    return NO;
+  else
+    return YES;
+}
+
+- (NSString *) sogoSecretValue
+{
+  NSString *value, *type;
+  NSDictionary *env;
+
+  type = [self stringForKey: @"SOGoSecretType"];
+  if(!type)
+    type = @"none";
+
+  if ([type isEqualToString:@"plain"])
+  {
+    value = [self stringForKey: @"SOGoSecretValue"];
+  }
+  else if ([type isEqualToString:@"env"])
+  {
+    value = [self stringForKey: @"SOGoSecretValue"];
+    [self errorWithFormat: @"SOGo env fetching %@", value];
+    if(!value || [value length] < 1)
+    {
+      [self errorWithFormat: @"SOGoSecretValue is not set!"];
+      return nil;
+    }
+    env = [[NSProcessInfo processInfo] environment];
+    value = [env objectForKey:value];
+  }
+  else if ([type isEqualToString:@"none"])
+  {
+    return nil;
+  }
+  else {
+    [self errorWithFormat: @"SOGo can't understand the type of secret SOGoSecretType"];
+    return nil;
+  }
+
+  if(!value || [value length] != 32){
+    [self errorWithFormat: @"SOGo doesn't have a correct secret value of 32 chars SOGoSecretValue"];
+    return nil;
+  }
+
+  return value;
+}
+
+
 - (BOOL) useRelativeURLs
 {
   return [self boolForKey: @"WOUseRelativeURLs"];
@@ -467,6 +523,16 @@ _injectConfigurationFromFile (NSMutableDictionary *defaultsDict,
   return [self stringForKey: @"SOGoHelpURL"];
 }
 
+NSComparisonResult languageSort(id el1, id el2, void *context)
+{
+    NSString *t1, *t2;
+
+    t1 = [context labelForKey: el1];
+    t2 = [context labelForKey: el2];
+
+    return [t1 compare: t2 options: NSCaseInsensitiveSearch];
+}
+
 - (NSArray *) supportedLanguages
 {
   static NSArray *supportedLanguages = nil;
@@ -809,5 +875,63 @@ _injectConfigurationFromFile (NSMutableDictionary *defaultsDict,
   return disableSharing;
 }
 
+- (BOOL)isURLEncryptionEnabled {
+  return [self boolForKey: @"SOGoURLEncryptionEnabled"];
+}
+
+- (NSString *) urlEncryptionPassphrase
+{
+  NSString *passphrase;
+
+  passphrase = [self stringForKey: @"SOGoURLEncryptionPassphrase"];
+
+  if (!passphrase)
+    passphrase = @"SOGoSuperSecret0"; // Default passphrase
+
+  return passphrase;
+}
+
+- (NSArray *) disableSharingAnyAuthUser
+{
+  static NSArray *disableSharingAnyAuthUser = nil;
+
+  if (!disableSharingAnyAuthUser)
+    {
+      disableSharingAnyAuthUser = [self stringArrayForKey: @"SOGoDisableSharingAnyAuthUser"];
+      [disableSharingAnyAuthUser retain];
+    }
+  
+  return disableSharingAnyAuthUser;
+}
+
+- (NSArray *) disableExport
+{
+  static NSArray *disableExport = nil;
+
+  if (!disableExport)
+    {
+      disableExport = [self stringArrayForKey: @"SOGoDisableExport"];
+      [disableExport retain];
+    }
+  
+  return disableExport;
+}
+
+- (BOOL) globalAddressBookFirstEntriesEnabled
+{
+  return [self boolForKey: @"SOGoGlobalAddressBookFirstEntries"];
+}
+
+- (int) globalAddressBookFirstEntriesCount
+{
+  int v;
+
+  v = [self integerForKey: @"SOGoGlobalAddressBookFirstEntriesCount"];
+
+  if (!v)
+    v = 100;
+
+  return v;
+}
 
 @end
