@@ -39,6 +39,7 @@
 #import "SOGoUserManager.h"
 #import "SOGoUserSettings.h"
 #import "WOResourceManager+SOGo.h"
+#import "NSString+Crypto.h"
 
 #if defined(MFA_CONFIG)
 #include <liboath/oath.h>
@@ -46,6 +47,8 @@
 
 #import "SOGoUser.h"
 
+static const NSString *kEncryptedUserNamePrefix = @"uenc";
+
 @implementation SoUser (SOGoExtension)
 
 - (SOGoDomainDefaults *) userDefaults
@@ -647,16 +650,17 @@
 
 - (void) _appendSystemMailAccountWithDelegatedIdentities: (BOOL) appendDeletegatedIdentities
 {
-  NSString *fullName, *imapLogin, *imapServer, *cImapServer,
-    *encryption, *scheme, *action, *queryTls, *customEmail, *sieveServer, *tlsVerifyMode;
+  NSString *fullName, *imapLogin, *imapServer, *cImapServer, *smtpServer,
+    *imapEncryption, *smtpEncryption, *imapScheme, *smtpScheme, *action, *imapQueryTls, *smtpQueryTls, 
+    *customEmail, *sieveServer, *imapTlsVerifyMode, *smtpTlsVerifyMode;
   NSMutableDictionary *mailAccount, *identity, *mailboxes, *receipts, *security, *mailSettings;
-  NSDictionary *queryComponents;
-  NSNumber *port;
+  NSDictionary *imapQueryComponents, *smtpQueryComponents;
+  NSNumber *imapPort, *smtpPort;
   NSMutableArray *identities, *mails;
   NSArray *delegators, *delegates;
-  NSURL *url, *cUrl;
+  NSURL *imapUrl, *cImapUrl, *smtpUrl;
   unsigned int count, max; //, default_identity;
-  NSInteger defaultPort;
+  NSInteger imapDefaultPort, smtpDefaultPort;
   NSUInteger index;
   BOOL hasDefaultIdentity;
 
@@ -672,7 +676,7 @@
                                    inDomain: [self domain]];
   [mailAccount setObject: imapLogin forKey: @"userName"];
 
-  // 2. server
+  // 2.1 IMAP server
   // imapServer might have the following format
   // localhost
   // localhost:143
@@ -684,57 +688,105 @@
 
   cImapServer = [self _fetchFieldForUser: @"c_imaphostname"];
   imapServer = [[self domainDefaults] imapServer];
-  cUrl = [NSURL URLWithString: (cImapServer ? cImapServer : @"")];
-  url = [NSURL URLWithString: imapServer];
-  if([cUrl host])
-    imapServer = [cUrl host];
+  cImapUrl = [NSURL URLWithString: (cImapServer ? cImapServer : @"")];
+  imapUrl = [NSURL URLWithString: imapServer];
+  if([cImapUrl host])
+    imapServer = [cImapUrl host];
   else
     if(cImapServer)
       imapServer = cImapServer;
     else
-      if([url host])
-        imapServer = [url host];
+      if([imapUrl host])
+        imapServer = [imapUrl host];
   [mailAccount setObject: imapServer forKey: @"serverName"];
 
-  // 3. port & encryption
-  scheme = [cUrl scheme] ? [cUrl scheme] : [url scheme];
-  queryComponents = [cUrl query] ? [cUrl queryComponents] : [url queryComponents];
-  queryTls = [queryComponents valueForKey: @"tls"];
-  tlsVerifyMode = [queryComponents valueForKey: @"tlsVerifyMode"];
+  // 2.2 IMAP port & encryption
+  imapScheme = [cImapUrl scheme] ? [cImapUrl scheme] : [imapUrl scheme];
+  imapQueryComponents = [cImapUrl query] ? [cImapUrl queryComponents] : [imapUrl queryComponents];
+  imapQueryTls = [imapQueryComponents valueForKey: @"tls"];
+  imapTlsVerifyMode = [imapQueryComponents valueForKey: @"tlsVerifyMode"];
 
-  if (!tlsVerifyMode)
-    tlsVerifyMode = @"default";
+  if (!imapTlsVerifyMode)
+    imapTlsVerifyMode = @"default";
 
-  if (scheme
-      && [scheme caseInsensitiveCompare: @"imaps"] == NSOrderedSame)
+  if (imapScheme && [imapScheme caseInsensitiveCompare: @"imaps"] == NSOrderedSame)
     {
-      if (queryTls && [queryTls caseInsensitiveCompare: @"YES"] == NSOrderedSame)
+      if (imapQueryTls && [imapQueryTls caseInsensitiveCompare: @"YES"] == NSOrderedSame)
         {
-          defaultPort = 143;
-          encryption = @"tls";
+          imapDefaultPort = 143;
+          imapEncryption = @"tls";
         }
       else
         {
-          encryption = @"ssl";
-          defaultPort = 993;
+          imapEncryption = @"ssl";
+          imapDefaultPort = 993;
         }
     }
   else
     {
-      if (queryTls && [queryTls caseInsensitiveCompare: @"YES"] == NSOrderedSame)
-        encryption = @"tls";
+      if (imapQueryTls && [imapQueryTls caseInsensitiveCompare: @"YES"] == NSOrderedSame)
+        imapEncryption = @"tls";
       else
-        encryption = @"none";
+        imapEncryption = @"none";
 
-      defaultPort = 143;
+      imapDefaultPort = 143;
     }
 
-  port = [cUrl port] ? [cUrl port] : [url port];
-  if ([port intValue] == 0) /* port is nil or intValue == 0 */
-    port = [NSNumber numberWithInt: defaultPort];
-  [mailAccount setObject: port forKey: @"port"];
-  [mailAccount setObject: encryption forKey: @"encryption"];
-  [mailAccount setObject: tlsVerifyMode forKey: @"tlsVerifyMode"];
+  imapPort = [cImapUrl port] ? [cImapUrl port] : [imapUrl port];
+  if ([imapPort intValue] == 0) /* port is nil or intValue == 0 */
+    imapPort = [NSNumber numberWithInt: imapDefaultPort];
+  [mailAccount setObject: imapPort forKey: @"port"];
+  [mailAccount setObject: imapEncryption forKey: @"encryption"];
+  [mailAccount setObject: imapTlsVerifyMode forKey: @"tlsVerifyMode"];
+
+  //3.1 SMTP server
+  smtpServer = [[self domainDefaults] smtpServer];
+  smtpUrl = [NSURL URLWithString: smtpServer];
+  if([smtpUrl host])
+    smtpServer = [smtpUrl host];
+  [mailAccount setObject: smtpServer forKey: @"smtpServerName"];
+
+  // 3.2 SMTP port and encryption
+  smtpScheme = [smtpUrl scheme];
+  smtpQueryComponents = [smtpUrl queryComponents];
+  smtpQueryTls = [smtpQueryComponents valueForKey: @"tls"];
+  smtpTlsVerifyMode = [smtpQueryComponents valueForKey: @"tlsVerifyMode"];
+
+  if (!smtpTlsVerifyMode)
+    smtpTlsVerifyMode = @"default";
+
+  if (smtpScheme && [smtpScheme caseInsensitiveCompare: @"smtps"] == NSOrderedSame)
+    {
+      if (smtpQueryTls && [smtpQueryTls caseInsensitiveCompare: @"YES"] == NSOrderedSame)
+        {
+          smtpDefaultPort = 465; //Shoud be 587 but sope sope-mime/NGMail/NGSmtpClient.m initWithUrl doesn't agree...
+          smtpEncryption = @"tls";
+        }
+      else
+        {
+          smtpEncryption = @"ssl";
+          smtpDefaultPort = 465; //Shoud be 587 but sope sope-mime/NGMail/NGSmtpClient.m initWithUrl doesn't agree...
+        }
+    }
+  else
+    {
+      if (smtpQueryTls && [smtpQueryTls caseInsensitiveCompare: @"YES"] == NSOrderedSame)
+      {
+        smtpEncryption = @"tls";
+        smtpDefaultPort = 465;
+      }
+      else {
+        smtpEncryption = @"none";
+        smtpDefaultPort = 25;
+      }
+    }
+
+  smtpPort = [smtpUrl port];
+  if ([smtpPort intValue] == 0) /* port is nil or intValue == 0 */
+    smtpPort = [NSNumber numberWithInt: smtpDefaultPort];
+  [mailAccount setObject: smtpPort forKey: @"smtpPort"];
+  [mailAccount setObject: smtpEncryption forKey: @"smtpEncryption"];
+  [mailAccount setObject: smtpTlsVerifyMode forKey: @"smtpTlsVerifyMode"];
 
   // 4. Sieve server
   sieveServer = [self _fetchFieldForUser: @"c_sievehostname"];
@@ -989,7 +1041,53 @@
         {
           auxAccounts = [[self userDefaults] auxiliaryMailAccounts];
           if (auxAccounts)
+          {
+            //Check if we need to decrypt password
+            NSString* sogoSecret;
+            sogoSecret = [[SOGoSystemDefaults sharedSystemDefaults] sogoSecretValue];
+            if(sogoSecret)
+            {
+              int i;
+              NSString *encryptedPassword, *password, *iv, *tag;
+              NSDictionary *account, *accountPassword;
+              NSException* exception = nil;
+              for (i = 0; i < [auxAccounts count]; i++)
+              {
+                account = [auxAccounts objectAtIndex: i];
+                if (![[account objectForKey: @"password"] isKindOfClass: [NSDictionary class]])
+                {
+                  [self errorWithFormat:@"Can't decrypt the password for auxiliary account %@, is not a dictionnary",
+                              [account objectForKey: @"name"]];
+                  continue;
+                }
+
+                accountPassword = [account objectForKey: @"password"];
+                encryptedPassword = [accountPassword objectForKey: @"cypher"];
+                iv = [accountPassword objectForKey: @"iv"];
+                tag = [accountPassword objectForKey: @"tag"];
+                NS_DURING
+                {
+                  password = [encryptedPassword decryptAES256GCM: sogoSecret iv: iv tag: tag exception:&exception];
+                }
+                NS_HANDLER
+                {
+                  [self errorWithFormat:@"Can't decrypt the password for auxiliary account %@",
+                              [account objectForKey: @"name"]];
+                  password = [account objectForKey: @"password"];
+                }
+                NS_ENDHANDLER
+                if(exception)
+                  [self errorWithFormat:@"Can't decrypt the password for auxiliary account %@: %@",
+                              [account objectForKey: @"name"], [exception reason]];
+                else if(!password)
+                  [self errorWithFormat:@"No exception but decyrpted password is empty for account %@",[account objectForKey: @"name"]];
+                else
+                  [account setObject: password forKey: @"password"];
+              }
+            }
+
             [mailAccounts addObjectsFromArray: auxAccounts];
+          }
         }
     }
 
@@ -1060,7 +1158,7 @@
 
 - (SOGoAppointmentFolder *) personalCalendarFolderInContext: (WOContext *) context
 {
-  return [[self calendarsFolderInContext: context] lookupPersonalFolder: [[context request] isMacOSXVenturaCalendarApp] ? @"PERSONAL" : @"personal"
+  return [[self calendarsFolderInContext: context] lookupPersonalFolder: @"personal"
                                                          ignoringRights: YES];
 }
 
@@ -1197,4 +1295,80 @@
   return [accessValue boolValue];
 }
 
+/* Encryption */
++ (NSString *) getEncryptedUsernameIfNeeded:(NSString *)username request: (WORequest *)request
+{
+  NSException *exception;
+  NSString *tmp, *cacheKey;
+  SOGoCache *cache;
+
+  if (![[SOGoSystemDefaults sharedSystemDefaults] isURLEncryptionEnabled] || [username isEqualToString: @"anonymous"] || [[request requestHandlerKey] isEqualToString:@"dav"])
+    return username;
+
+  cache = [SOGoCache sharedCache];
+  cacheKey = [NSString stringWithFormat: @"%@_%@", kEncryptedUserNamePrefix, username];
+  
+  exception = nil;
+  tmp = nil;
+
+  tmp = [cache valueForKey: cacheKey];
+  if (tmp) {
+    return tmp;
+  } else {
+    if (username && [username length] > 0) {
+      tmp = [username encodeAES128ECBBase64: [[SOGoSystemDefaults sharedSystemDefaults] urlEncryptionPassphrase] encodedURL:YES exception: &exception];
+      if (!exception) {
+        [cache setValue:tmp forKey:cacheKey];
+        return tmp;
+      } else {
+        [self errorWithFormat: @"URL Encryption error : %@", [exception reason]];
+        return username;
+      }
+    } else {
+      [self logWithFormat: @"Empty username for encryption"];
+      return username;
+    }
+  }
+}
+
++ (NSString *) getDecryptedUsernameIfNeeded:(NSString *)username request: (WORequest *)request
+{
+  NSException *exception;
+  NSString *tmp, *cacheKey;
+  SOGoCache *cache;
+
+  if (![[SOGoSystemDefaults sharedSystemDefaults] isURLEncryptionEnabled] || [username isEqualToString: @"anonymous"] || [[request requestHandlerKey] isEqualToString:@"dav"])
+    return username;
+
+  cache = [SOGoCache sharedCache];
+  cacheKey = [NSString stringWithFormat: @"%@_%@", kEncryptedUserNamePrefix, username];
+  exception = nil;
+  tmp = nil;
+  
+  tmp = [cache valueForKey: cacheKey];
+  if (tmp) {
+    return tmp;
+  } else {
+    if (username && [username length] > 0) {
+      tmp = [username decodeAES128ECBBase64: [[SOGoSystemDefaults sharedSystemDefaults] urlEncryptionPassphrase] encodedURL:YES exception: &exception];
+      if (tmp) {
+        [cache setValue:tmp forKey:cacheKey];
+      } else {
+        [cache setValue:username forKey:cacheKey];
+      }
+      if (!exception) {
+        return tmp;
+      } else {
+        [self errorWithFormat: @"URL Decryption error : %@", [exception reason]];
+        return username;
+      }
+    } else {
+      [self errorWithFormat: @"Empty username for decryption"];
+      return username;
+    }
+    
+  }
+}
+
+
 @end /* SOGoUser */
