@@ -29,10 +29,11 @@
 #import <NGImap4/NGSieveClient.h>
 
 #import <SOPE/NGCards/iCalRecurrenceRule.h>
-
+#import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/NSArray+Utilities.h>
 #import <SOGo/NSObject+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
+#import <SOGo/NSString+Crypto.h>
 #import <SOGo/SOGoDomainDefaults.h>
 #import <SOGo/SOGoSieveManager.h>
 #import <SOGo/SOGoUser.h>
@@ -78,6 +79,32 @@ static SoProduct *preferencesProduct = nil;
   return labelsDictionary;
 }
 
+- (NSDictionary *) _localizedContactsLabels
+{
+  NSArray *categoryLabels, *localizedCategoryLabels;
+  NSDictionary *labelsDictionary;
+
+  labelsDictionary = nil;
+  localizedCategoryLabels = [[self labelForKey: @"contacts_category_labels"
+                                   withResourceManager: [preferencesProduct resourceManager]]
+                              componentsSeparatedByString: @","];
+  categoryLabels = [[[preferencesProduct resourceManager]
+                                      stringForKey: @"contacts_category_labels"
+                                      inTableNamed: nil
+                                  withDefaultValue: @""
+                                         languages: [NSArray arrayWithObject: @"English"]]
+                                      componentsSeparatedByString: @","];
+
+  if ([localizedCategoryLabels count] == [categoryLabels count])
+    labelsDictionary = [NSDictionary dictionaryWithObjects: localizedCategoryLabels
+                                                   forKeys: categoryLabels];
+  else
+    [self logWithFormat: @"ERROR: localizable strings contacts_category_labels is incorrect for language %@",
+          [[[context activeUser] userDefaults] language]];
+
+  return labelsDictionary;
+}
+
 - (WOResponse *) activeExternalSieveScriptsAction
 {
   NGSieveClient *client;
@@ -121,7 +148,7 @@ static SoProduct *preferencesProduct = nil;
   SOGoUserDefaults *defaults;
   SOGoDomainDefaults *domainDefaults;
   NSMutableArray *accounts;
-  NSDictionary *categoryLabels, *vacationOptions;
+  NSDictionary *calendarCategoryLabels, *contactsCategoriesLabels, *vacationOptions;
 
   if (!preferencesProduct)
     {
@@ -131,7 +158,8 @@ static SoProduct *preferencesProduct = nil;
 
   defaults = [[context activeUser] userDefaults];
   domainDefaults = [[context activeUser] domainDefaults];
-  categoryLabels = nil;
+  calendarCategoryLabels = nil;
+  contactsCategoriesLabels = nil;
 
   //
   // Default General preferences
@@ -223,7 +251,7 @@ static SoProduct *preferencesProduct = nil;
     {
       NSArray *defaultCalendarCategories;
 
-      categoryLabels = [self _localizedCategoryLabels];
+      calendarCategoryLabels = [self _localizedCategoryLabels];
 
       if ((defaultCalendarCategories = [defaults calendarCategories]))
         {
@@ -239,7 +267,7 @@ static SoProduct *preferencesProduct = nil;
           for (count = 0; count < max; count++)
             {
               label = [defaultCalendarCategories objectAtIndex: count];
-              if (!(localizedLabel = [categoryLabels objectForKey: label]))
+              if (!(localizedLabel = [calendarCategoryLabels objectForKey: label]))
                 {
                   localizedLabel = label;
                 }
@@ -252,15 +280,15 @@ static SoProduct *preferencesProduct = nil;
         {
           // Calendar categories are taken from localizable strings
 
-          [defaults setCalendarCategories: [categoryLabels allValues]];
+          [defaults setCalendarCategories: [calendarCategoryLabels allValues]];
         }
     }
   if (![[defaults source] objectForKey: @"SOGoCalendarCategoriesColors"])
     {
       NSDictionary *defaultCalendarCategoriesColors;
 
-      if (!categoryLabels)
-        categoryLabels = [self _localizedCategoryLabels];
+      if (!calendarCategoryLabels)
+        calendarCategoryLabels = [self _localizedCategoryLabels];
 
       if ((defaultCalendarCategoriesColors = [defaults calendarCategoriesColors]))
         {
@@ -278,7 +306,7 @@ static SoProduct *preferencesProduct = nil;
           for (count = 0; count < max; count++)
             {
               label = [defaultCalendarCategories objectAtIndex: count];
-              if (!(localizedLabel = [categoryLabels objectForKey: label]))
+              if (!(localizedLabel = [calendarCategoryLabels objectForKey: label]))
                 {
                   localizedLabel = label;
                 }
@@ -296,7 +324,7 @@ static SoProduct *preferencesProduct = nil;
           NSMutableDictionary *colors;
           int i;
 
-          calendarCategories = [categoryLabels allValues];
+          calendarCategories = [calendarCategoryLabels allValues];
           colors = [NSMutableDictionary dictionaryWithCapacity: [calendarCategories count]];
 
           for (i = 0; i < [calendarCategories count]; i++)
@@ -311,18 +339,41 @@ static SoProduct *preferencesProduct = nil;
   //
   // Populate default contact categories, based on the user's preferred language
   //
-  if (![defaults contactsCategories])
+  if (![[defaults source] objectForKey: @"SOGoContactsCategories"])
     {
       NSArray *contactsCategories;
 
-      contactsCategories = [[[[self labelForKey: @"contacts_category_labels"  withResourceManager: [preferencesProduct resourceManager]]
-                              componentsSeparatedByString: @","] trimmedComponents]
-                             sortedArrayUsingSelector: @selector (localizedCaseInsensitiveCompare:)];
+      contactsCategoriesLabels = [self _localizedContactsLabels];
 
-      if (!contactsCategories)
-        contactsCategories = [NSArray array];
+      if((contactsCategories = [defaults contactsCategories]))
+      {
+        // Contact categories are taken from SOGo's configuration or SOGoDefaults.plist
 
-      [defaults setContactsCategories: contactsCategories];
+        NSMutableArray *filteredContactsCategories;
+        NSString *label, *localizedLabel;
+        int count, max;
+
+        max = [contactsCategories count];
+        filteredContactsCategories = [NSMutableArray arrayWithCapacity: max];
+
+        for (count = 0; count < max; count++)
+          {
+            label = [contactsCategories objectAtIndex: count];
+            if (!(localizedLabel = [contactsCategoriesLabels objectForKey: label]))
+              {
+                localizedLabel = label;
+              }
+            [filteredContactsCategories addObject: localizedLabel];
+          }
+
+        [defaults setContactsCategories: filteredContactsCategories];
+      }
+      else
+      {
+        // Contact categories are taken from localizable strings
+
+        [defaults setContactsCategories: [contactsCategoriesLabels allValues]];
+      }
     }
 
   if (![[defaults source] objectForKey: @"SOGoMailAddOutgoingAddresses"])
@@ -343,8 +394,21 @@ static SoProduct *preferencesProduct = nil;
   if (![[defaults source] objectForKey: @"SOGoMailReplyPlacement"])
     [[defaults source] setObject: [defaults mailReplyPlacement] forKey: @"SOGoMailReplyPlacement"];
 
-  if (![[defaults source] objectForKey: @"SOGoMailSignaturePlacement"])
-    [[defaults source] setObject: [defaults mailSignaturePlacement] forKey: @"SOGoMailSignaturePlacement"];
+  if (![[defaults source] objectForKey: @"SOGoMailUseSignatureOnNew"])
+    [[defaults source] setObject: [NSNumber numberWithBool: [defaults mailUseSignatureOnNew]] forKey: @"SOGoMailUseSignatureOnNew"];
+  
+  if (![[defaults source] objectForKey: @"SOGoMailUseSignatureOnReply"])
+    [[defaults source] setObject: [NSNumber numberWithBool: [defaults mailUseSignatureOnReply]] forKey: @"SOGoMailUseSignatureOnReply"];
+  
+  if (![[defaults source] objectForKey: @"SOGoMailUseSignatureOnForward"])
+    [[defaults source] setObject: [NSNumber numberWithBool: [defaults mailUseSignatureOnForward]] forKey: @"SOGoMailUseSignatureOnForward"];
+  
+  if (![[defaults source] objectForKey: @"SOGoMailAddOutgoingAddresses"])
+    [[defaults source] setObject: [NSNumber numberWithBool: [defaults mailAddOutgoingAddresses]] forKey: @"SOGoMailAddOutgoingAddresses"];
+
+
+  if (![[defaults source] objectForKey: @"SOGoMailDisplayFullEmail"])
+    [[defaults source] setObject: [NSNumber numberWithBool: [defaults mailDisplayFullEmail]]  forKey: @"SOGoMailDisplayFullEmail"];
 
   if (![[defaults source] objectForKey: @"SOGoMailComposeMessageType"])
     [[defaults source] setObject: [defaults mailComposeMessageType] forKey: @"SOGoMailComposeMessageType"];
@@ -358,6 +422,9 @@ static SoProduct *preferencesProduct = nil;
   if ([[defaults source] objectForKey: @"SOGoMailAutoMarkAsReadDelay"] == nil)
     [[defaults source] setObject: [NSNumber numberWithInt: [defaults mailAutoMarkAsReadDelay]] forKey: @"SOGoMailAutoMarkAsReadDelay"];
 
+  if ([[defaults source] objectForKey: @"SOGoMailHideInlineAttachments"] == nil)
+    [[defaults source] setObject: [NSNumber numberWithBool: [defaults hideInlineAttachments]] forKey: @"SOGoMailHideInlineAttachments"];
+  
   if (![[defaults source] objectForKey: @"SOGoMailAutoSave"])
     [[defaults source] setObject: [defaults mailAutoSave] forKey: @"SOGoMailAutoSave"];
 
@@ -385,9 +452,14 @@ static SoProduct *preferencesProduct = nil;
     }
 
   if ([domainDefaults forwardEnabled] && ![defaults forwardOptions])
-    {
-      [defaults setForwardOptions: [NSDictionary new]];
-    }
+  {
+    [defaults setForwardOptions: [NSDictionary new]];
+  }
+
+  if ([domainDefaults notificationEnabled] && ![defaults notificationOptions])
+  {
+    [defaults setNotificationOptions: [NSDictionary new]];
+  }
 
   if ([[defaults source] dirty])
     [defaults synchronize];
@@ -422,8 +494,10 @@ static SoProduct *preferencesProduct = nil;
   if ([accounts count])
     {
       int i;
-      NSDictionary *security;
+      NSDictionary *security, *accountPassword;
       NSMutableDictionary *auxAccount, *limitedSecurity;
+      NSString *password, *encryptedPassword, *sogoSecret, *iv, *tag;
+      NSException* exception = nil;
 
       for (i = 0; i < [accounts count]; i++)
         {
