@@ -35,6 +35,7 @@
 #import <SOGo/NSArray+Utilities.h>
 #import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
+#import <SOGo/NSString+Crypto.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserSettings.h>
 #import <SOGo/SOGoSieveManager.h>
@@ -136,13 +137,20 @@ static NSArray *reminderValues = nil;
             vacationOptions = [NSMutableDictionary new];
 	}
 
-      if ([dd forwardEnabled])
+  if ([dd forwardEnabled])
 	{
 	  forwardOptions = [[userDefaults forwardOptions] mutableCopy];
 	  if (!forwardOptions)
             forwardOptions = [NSMutableDictionary new];
 	}
 
+  if ([dd notificationEnabled])
+	{
+	  notificationOptions = [[userDefaults notificationOptions] mutableCopy];
+	  if (!notificationOptions)
+            notificationOptions = [NSMutableDictionary new];
+	}
+
       mailCustomFromEnabled = [dd mailCustomFromEnabled];
       forwardEnabled = [dd forwardEnabled];
 
@@ -160,6 +168,7 @@ static NSArray *reminderValues = nil;
   [sieveFilters release];
   [vacationOptions release];
   [forwardOptions release];
+  [notificationOptions release];
   [daysOfWeek release];
   [addressBooksIDWithDisplayName release];
   [client release];
@@ -901,18 +910,63 @@ static NSArray *reminderValues = nil;
 //
 // Used by templates
 //
+- (NSDictionary *) _localizedContactsLabels
+{
+  NSArray *categoryLabels, *localizedCategoryLabels;
+  NSDictionary *labelsDictionary;
+
+  labelsDictionary = nil;
+  localizedCategoryLabels = [[self labelForKey: @"contacts_category_labels"
+                                   withResourceManager: [self resourceManager]]
+                              componentsSeparatedByString: @","];
+  categoryLabels = [[[self resourceManager]
+                                      stringForKey: @"contacts_category_labels"
+                                      inTableNamed: nil
+                                  withDefaultValue: @""
+                                         languages: [NSArray arrayWithObject: @"English"]]
+                                      componentsSeparatedByString: @","];
+
+  if ([localizedCategoryLabels count] == [categoryLabels count])
+    labelsDictionary = [NSDictionary dictionaryWithObjects: localizedCategoryLabels
+                                                   forKeys: categoryLabels];
+  else
+    [self logWithFormat: @"ERROR: localizable strings contacts_category_labels is incorrect for language %@",
+          [[[context activeUser] userDefaults] language]];
+
+  return labelsDictionary;
+}
+
 - (NSString *) defaultContactsCategories
 {
-  NSArray *contactsCategories;
+  NSArray *labels;
+  NSDictionary *localizedLabels;
+  NSMutableArray *defaultCategoriesLabels;
+  NSString *label, *localizedLabel;
+  unsigned int i;
 
-  contactsCategories = [[[[self labelForKey: @"contacts_category_labels"  withResourceManager: [self resourceManager]]
-                              componentsSeparatedByString: @","] trimmedComponents]
-                             sortedArrayUsingSelector: @selector (localizedCaseInsensitiveCompare:)];
+  localizedLabels = [self _localizedContactsLabels];
+  labels = [[SOGoSystemDefaults sharedSystemDefaults] contactsCategories];
+  defaultCategoriesLabels = [NSMutableArray array];
 
-  if (!contactsCategories)
-    contactsCategories = [NSArray array];
+  if(labels)
+  {
+    for (i = 0; i < [labels count]; i++)
+    {
+      label = [labels objectAtIndex: i];
+      if (!(localizedLabel = [localizedLabels objectForKey: label]))
+      {
+        localizedLabel = label;
+      }
+      [defaultCategoriesLabels addObject: localizedLabel];
+    }
+  }
+  else
+  {
+    defaultCategoriesLabels = [localizedLabels allValues];
+  }
 
-  return [contactsCategories jsonRepresentation];
+
+  return [defaultCategoriesLabels jsonRepresentation];
 }
 
 //
@@ -989,6 +1043,15 @@ static NSArray *reminderValues = nil;
   return [domains jsonRepresentation];
 }
 
+/* mail notifications */
+//
+// Used by templates
+//
+- (BOOL) isNotificationEnabled
+{
+  return [[user domainDefaults] notificationEnabled];
+}
+
 //
 // Used by templates
 //
@@ -1129,12 +1192,13 @@ static NSArray *reminderValues = nil;
            userCanChangePassword];
 }
 
+
 //
 // Used by wox template
 //
 - (NSArray *) languages
 {
-  return [[SOGoSystemDefaults sharedSystemDefaults] supportedLanguages];
+  return [[[SOGoSystemDefaults sharedSystemDefaults] supportedLanguages] sortedArrayUsingFunction: languageSort context: self];
 }
 
 //
@@ -1295,6 +1359,7 @@ static NSArray *reminderValues = nil;
   if (!knownKeys)
     {
       knownKeys = [NSArray arrayWithObjects: @"id", @"name", @"serverName", @"port",
+                           @"smtpServerName", @"smtpPort", @"smtpEncryption", @"smtpAuth",
                            @"userName", @"password", @"encryption", @"replyTo",
                            @"identities", @"mailboxes", @"forceDefaultIdentity",
                            @"receipts", @"security", @"isNew",
@@ -1340,15 +1405,18 @@ static NSArray *reminderValues = nil;
 //
 // Used internally
 //
-- (void) _extractMainAccountSettings: (NSDictionary *) account
+- (NSException *) _extractMainAccountSettings: (NSDictionary *) account
                         inDictionary: (NSMutableDictionary *) target
 {
   NSArray *identities;
   SOGoDomainDefaults *dd;
   NSMutableArray *previousIdentities, *newIdentities;
-  NSMutableDictionary *identity, *newIdentitiesAsDict;
+  NSMutableDictionary *identity, *previousIdentity, *newIdentitiesAsDict;
+  NSException *error;
   int i;
-  BOOL isDefault;
+  BOOL isDefault, found;
+
+  error = nil;
 
   if ([account isKindOfClass: [NSDictionary class]])
     {
@@ -1367,6 +1435,37 @@ static NSArray *reminderValues = nil;
             newIdentitiesAsDict = [[NSMutableDictionary alloc] init];
             for (NSDictionary *identity in identities) {
               [newIdentitiesAsDict setObject: identity forKey: [identity objectForKey:@"email"]];
+              // Check if email has been changed (unauthorized)
+              if ([identity objectForKey:@"email"]) {
+                found = NO;
+                for (previousIdentity in previousIdentities) {
+                  if ([[identity objectForKey:@"email"] isEqualToString: [previousIdentity objectForKey:@"email"]]) {
+                    found = YES;
+                  }
+                }
+                if (!found) {
+                  error = [NSException exceptionWithName: @"SOGOPreferencesException"
+                                      reason: @"Invalid operation"
+                                    userInfo: nil];;
+                  return error; // Break
+                }
+              }
+              
+              // Check if replyTo has been changed (unauthorized)
+              if ([identity objectForKey:@"replyTo"]) {
+                found = NO;
+                for (previousIdentity in previousIdentities) {
+                  if ([[identity objectForKey:@"replyTo"] isEqualToString: [previousIdentity objectForKey:@"replyTo"]]) {
+                    found = YES;
+                  }
+                }
+                if (!found) {
+                  error = [NSException exceptionWithName: @"SOGOPreferencesException"
+                                      reason: @"Invalid operation"
+                                    userInfo: nil];;
+                  return error; // Break
+                }
+              }
             }
 
             i = 0;
@@ -1392,10 +1491,14 @@ static NSArray *reminderValues = nil;
             for (identity in previousIdentities) {
               if ([newIdentitiesAsDict objectForKey: [identity objectForKey:@"email"]]) {
                   [identity setObject: [[newIdentitiesAsDict objectForKey: [identity objectForKey:@"email"]] objectForKey:@"fullName"] forKey: @"fullName"];
-                  if ([identity objectForKey:@"signature"]) {
+                  if (newIdentitiesAsDict 
+                      && [newIdentitiesAsDict objectForKey: [identity objectForKey:@"email"]] 
+                      && [[newIdentitiesAsDict objectForKey: [identity objectForKey:@"email"]] objectForKey:@"signature"]) {
                     [identity setObject: [[newIdentitiesAsDict objectForKey: [identity objectForKey:@"email"]] objectForKey:@"signature"] forKey: @"signature"];
                   }
-                  if ([[newIdentitiesAsDict objectForKey: [identity objectForKey:@"email"]] objectForKey:@"isDefault"]) {
+                  if (newIdentitiesAsDict 
+                      && [newIdentitiesAsDict objectForKey: [identity objectForKey:@"email"]] 
+                      && [[newIdentitiesAsDict objectForKey: [identity objectForKey:@"email"]] objectForKey:@"isDefault"]) {
                     [identity setObject: [NSNumber numberWithBool: YES] forKey: @"isDefault"];
                   } else {
                     [identity setObject: [NSNumber numberWithBool: NO] forKey: @"isDefault"];
@@ -1418,6 +1521,8 @@ static NSArray *reminderValues = nil;
       [self _extractMainReceiptsPreferences: [account objectForKey: @"receipts"]  inDictionary: target];
       [self _extractMainSecurityPreferences: [account objectForKey: @"security"]  inDictionary: target];
     }
+
+    return error;
 }
 
 //
@@ -1428,6 +1533,7 @@ static NSArray *reminderValues = nil;
   int count, max;
   NSMutableArray *auxAccounts;
   NSMutableDictionary *account;
+  NSString *sogoSecret, *password;
 
   max = [accounts count];
   auxAccounts = [NSMutableArray arrayWithCapacity: max];
@@ -1438,6 +1544,24 @@ static NSArray *reminderValues = nil;
       if ([self _validateAccount: account])
         {
           [self _updateAuxiliaryAccount: account];
+
+          //Encrypt password if needed
+          sogoSecret = [[SOGoSystemDefaults sharedSystemDefaults] sogoSecretValue];
+          //Note that password here will always be a string and not a dictionnary of gcm encryption as it comes from the saveAction Request
+          password = [account objectForKey: @"password"];
+          if(sogoSecret && [password length] > 0)
+          {
+            NSDictionary* newPassword;
+            NSException* exception = nil;
+            newPassword = [password encryptAES256GCM: sogoSecret exception:&exception];
+            if(exception)
+              [self errorWithFormat:@"Can't encrypt the password: %@", [exception reason]];
+            else
+              [account setObject: newPassword forKey: @"password"];
+          }
+          else
+            [self warnWithFormat:@"Password for auxiliary accounts are not stored encrypted see SOGoSecretType"];
+
           [auxAccounts addObject: account];
         }
     }
@@ -1452,7 +1576,7 @@ static NSArray *reminderValues = nil;
   NSDictionary *oldAccount, *oldSecurity;
   NSEnumerator *comparisonAttributesList;
   NSMutableDictionary *newSecurity;
-  NSString *comparisonAttribute, *password, *certificate;
+  NSString *comparisonAttribute, *password, *certificate, *sogoSecret, *decryptedPassword;
 
   comparisonAttributes = [NSArray arrayWithObjects: @"serverName", @"userName", nil];
   oldAccounts = [user mailAccounts];
@@ -1476,11 +1600,61 @@ static NSArray *reminderValues = nil;
       // Use previous password if none is provided
       password = [newAccount objectForKey: @"password"];
       if (!password)
-        password = [oldAccount objectForKey: @"password"];
-      if (!password)
-        password = @"";
-      [newAccount setObject: password forKey: @"password"];
+      {
+        if([oldAccount objectForKey: @"password"])
+        {
+          if([[oldAccount objectForKey: @"password"] isKindOfClass: [NSDictionary class]])
+          {
+            //Old password is encrypted, decrypt it first as it will be encrypt again after
+            NSDictionary *accountPassword;
+            NSString *encryptedPassword, *iv, *tag;
+            accountPassword = [oldAccount objectForKey: @"password"];
+            encryptedPassword = [accountPassword objectForKey: @"cypher"];
+            iv = [accountPassword objectForKey: @"iv"];
+            tag = [accountPassword objectForKey: @"tag"];
+            sogoSecret = [[SOGoSystemDefaults sharedSystemDefaults] sogoSecretValue];
+            if(sogoSecret)
+            {
+              NSException* exception = nil;
+              NS_DURING
+                decryptedPassword = [encryptedPassword decryptAES256GCM: sogoSecret iv: iv tag: tag exception:&exception];
+                if(exception)
+                {
+                  [self errorWithFormat:@"Can't decrypt the password for auxiliary account %@: %@",
+                          [oldAccount objectForKey: @"name"], [exception reason]];
+                  decryptedPassword = @"";
+                }
+                else if(!decryptedPassword)
+                  [self errorWithFormat:@"No exception but decrypted password is empty for account %@",[oldAccount objectForKey: @"name"]];
+                else
+                  password = decryptedPassword;
+              NS_HANDLER
+                [self errorWithFormat:@"Can't decrypt the password for auxiliary account %@, probably wrong key",
+                            [oldAccount objectForKey: @"name"]];
+                decryptedPassword = @"";
+              NS_ENDHANDLER
+              [newAccount setObject: decryptedPassword forKey: @"password"];
+            }
+            else
+            {
+              [self errorWithFormat:@"Password currently stored for account %@ is encrypted but there is a no secret SOGoSecretValue to decrypt with",
+                            [newAccount objectForKey: @"name"]];
+              [newAccount setObject: @"" forKey: @"password"];
+            }
+          }
+          else
+          {
+            //Old password is not encrypted, nothing to do
+            [newAccount setObject: [oldAccount objectForKey: @"password"] forKey: @"password"];
+          }
+        }
+        else {
+          //No password for this account, weird choice but possible
+          [newAccount setObject: @"" forKey: @"password"];
+        }
+      }
 
+    
       // Keep previous certificate
       oldSecurity = [oldAccount objectForKey: @"security"];
       if (oldSecurity)
@@ -1510,6 +1684,44 @@ static NSArray *reminderValues = nil;
   return (forwardEnabled ? @"true" : @"false");
 }
 
+- (BOOL) doForwardsMatchTheConstraints: (NSArray *) forwardMails
+{
+  NSArray *allUserMails, *domainConstraints;
+  NSMutableArray *allUserDomains;
+  NSString *currentMail, *currentDomain, *userMail;
+  SOGoDomainDefaults *dd;
+  int constraint;
+  
+  dd = [[context activeUser] domainDefaults];
+  constraint = [dd forwardConstraints];
+
+  if(constraint > 0)
+  {
+    allUserMails = [[user allEmails] uniqueObjects];
+    allUserDomains = [NSMutableArray array];
+    for(userMail in allUserMails)
+    {
+      [allUserDomains push: [userMail mailDomain]];
+    }
+    for(currentMail in forwardMails)
+    {
+      currentDomain = [currentMail mailDomain];
+      domainConstraints = [dd forwardConstraintsDomains];
+      if (constraint == 1 && [allUserDomains indexOfObject: currentDomain] == NSNotFound)
+        return NO;
+      else if (constraint == 2 && [allUserDomains indexOfObject: currentDomain] != NSNotFound)
+        return NO;
+      else if (constraint == 2 && (!domainConstraints || [domainConstraints indexOfObject: currentDomain] == NSNotFound))
+        return NO;
+      else if (constraint == 3 && 
+              [allUserDomains indexOfObject: currentDomain] == NSNotFound &&
+                (!domainConstraints || [domainConstraints indexOfObject: currentDomain] == NSNotFound))
+        return NO;
+    }
+  }
+  return YES;
+}
+
 /**
  * @api {post} /so/:username/Preferences/save Save user's defaults and settings
  * @apiVersion 1.0.0
@@ -1539,8 +1751,8 @@ static NSArray *reminderValues = nil;
   if ((v = [o objectForKey: @"defaults"]))
     {
       NSMutableDictionary *sanitizedLabels;
-      NSArray *allKeys, *accounts, *identities;
-      NSDictionary *newLabels;
+      NSArray *allKeys, *accounts, *identities, *forwardMails;
+      NSDictionary *newLabels, *forwardPref;
       NSString *name;
       id loginModule;
 
@@ -1578,6 +1790,20 @@ static NSArray *reminderValues = nil;
           [v removeObjectForKey: @"SOGoAlternateAvatar"];
           [[[user userDefaults] source] removeObjectForKey: @"SOGoAlternateAvatar"];
         }
+      
+      //We check if there are forward constraints
+      forwardPref = [v objectForKey: @"Forward"];
+      if(forwardPref && [forwardPref isKindOfClass: [NSDictionary class]]
+                     && [forwardPref objectForKey: @"enabled"]
+                     && [[forwardPref objectForKey: @"enabled"] boolValue])
+      {
+        BOOL doForward = NO;
+        forwardMails = [forwardPref objectForKey: @"forwardAddress"];
+        if (forwardMails && [forwardMails isKindOfClass: [NSArray class]] && [forwardMails count]>0)
+          doForward = [self doForwardsMatchTheConstraints: [forwardPref objectForKey: @"forwardAddress"]];
+        if(!doForward)
+          [v removeObjectForKey: @"Forward"];
+      }
 
       if ([self userHasMailAccess])
         {
@@ -1636,11 +1862,15 @@ static NSArray *reminderValues = nil;
                   // - forceDefaultIdentity                => SOGoMailForceDefaultIdentity
                   // - receipts.receiptAction              => SOGoMailReceiptAllow
                   // - receipts.receiptNonRecipientAction  => SOGoMailReceiptNonRecipientAction
-                  // - receipts.receiptOutsideDomainAction => SOGoMailReceiptOutsideDomainAction
+                  // - receipts.receiptOutsideDomaforwardAddressinAction => SOGoMailReceiptOutsideDomainAction
                   // - receipts.receiptAnyAction           => SOGoMailReceiptAnyAction
                   // - security.alwaysSign                 => SOGoMailCertificateAlwaysSign
                   // - security.alwaysEncrypt              => SOGoMailCertificateAlwaysEncrypt
-                  [self _extractMainAccountSettings: [accounts objectAtIndex: 0]  inDictionary: v];
+                  if ([self _extractMainAccountSettings: [accounts objectAtIndex: 0]  inDictionary: v]) {
+                    return [self responseWithStatus: 403
+                              andString: @"Invalid operation"];
+                  }
+
                   if ([self mailAuxiliaryUserAccountsEnabled])
                     accounts = [self _extractAuxiliaryAccounts: accounts];
                   else
@@ -1726,7 +1956,8 @@ static NSArray *reminderValues = nil;
           dd = [[context activeUser] domainDefaults];
 
           // We check if the Sieve server is available *ONLY* if at least one of the option is enabled
-          if (!([dd sieveScriptsEnabled] || [dd vacationEnabled] || [dd forwardEnabled]) || [self _isSieveServerAvailable])
+          if (!([dd sieveScriptsEnabled] || [dd vacationEnabled] || [dd forwardEnabled] || [dd notificationEnabled]) 
+                  || [self _isSieveServerAvailable])
             {
               BOOL forceActivation = ![[v objectForKey: @"hasActiveExternalSieveScripts"] boolValue];
 
@@ -1743,6 +1974,9 @@ static NSArray *reminderValues = nil;
           else
             results = (id <WOActionResults>) [self responseWithStatus: 503
                          andJSONRepresentation: [NSDictionary dictionaryWithObjectsAndKeys: @"Service temporarily unavailable", @"message", nil]];
+        } else {
+          results = (id <WOActionResults>) [self responseWithStatus: 500
+                         andJSONRepresentation: [NSDictionary dictionaryWithObjectsAndKeys: @"Error during the validation", @"message", nil]];
         }
     }
 
@@ -1807,4 +2041,12 @@ static NSArray *reminderValues = nil;
   return ![[SOGoSystemDefaults sharedSystemDefaults] isEasUIDisabled];
 }
 
+- (BOOL) showCreateIdentity
+{
+  SOGoDomainDefaults *dd;
+  dd = [[context activeUser] domainDefaults];
+
+  return ![dd createIdentitiesDisabled];
+}
+
 @end
